// Entry
//
// Copyright (c) 2005 Satoshi Fukutomi <info@fuktommy.com>.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

//
// Get Element Value
//
function getValue(object, tagname) {
    var element = object.getElementsByTagName(tagname)[0];
    if (element) {
        return element.firstChild.nodeValue;
    } else {
        return null;
    }
}
// End of getValue


//
// Entry (An article)
//
var Entry = Class.create();
Entry.prototype = {
    initialize: function(item) {
        this.title = getValue(item, "title");
        this.link = getValue(item, "link");
        this.date = getValue(item, "dc:date") || getValue(item, "date");
        this.description = getValue(item, "description");
        this.content = getValue(item, "content:encoded") ||
                       getValue(item, "encoded");
        this.id = this.link.substring(this.link.search("#")+2)
    }
};

var DummyEntry = Class.create();
DummyEntry.prototype = {
    initialize: function() {
        this.title = "";
        this.link = "";
        this.date = "";
        this.description = "";
        this.content = "";
        this.id = "";
    }
};
// End of Entry


//
// Entry Cache
//
var EntryCache = Class.create();
EntryCache.prototype = {
    initialize: function() {
        this.cache = {};
        this.index = [];
    },

    append: function(entry) {
        if (this.cache[entry.id]) {
            if (! entry.content) {
                return;
            } else {
                this.index = this.index.without(entry.id);
            }
        }
        this.index.push(entry.id);
        this.cache[entry.id] = entry;
        while (this.index.length > cacheSize) {
            var id = this.index.shift();
            this.cache.remove(id);
        }
    }
};
// End of EntryCache
