#!/usr/local/bin/python
#
"""SLB Reader.
"""
#
# Copyright (c) 2005,2006 Satoshi Fukutomi <info@fuktommy.com>.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

#import cgitb; cgitb.enable()

import re
import os
import cgi
import time
import urllib
from xml.sax import parse, SAXParseException
from xml.sax.handler import ContentHandler

class Config(dict):
    def __init__(self, filename=None):
        dict.__init__(self)
        if filename:
            self.read(filename)

    def read(self, filename):
        for line in file(filename):
            key, val = line.split(":", 1)
            self[key.strip()] = val.strip()

    def getint(self, key):
        return int(self[key])

blogcgidir = '../../blog.fuktommy.com/admin'
basecgi = '/blog/'
config = Config('/virtual/fuktommy/private/blog.conf')

#blogcgidir = '../../blog/build/admin'
#basecgi = './?'
#config = Config('../../blog/build/admin/blog.conf')

class Item:
    '''RSS item'''

    def __init__(self, link='', title='', date='', creator='', license='',
                 description='', content=''):
        self.link  = link
        self.date  = date
        self.title = title
        self.creator = creator
        self.license = license
        self.description = description
        self.content = content.replace(']]>', '&#93;&#93;>')

# End of Item


class RSShandler(ContentHandler):
    elements = ['title', 'link', 'dc:date', 'dc:creator',
                'description', 'content:encoded']

    def __init__(self):
        ContentHandler.__init__(self)
        self.items = []
        self.state = ""

    def startElement(self, name, attrs):
        if name == "item":
            self.items.append(Item())
            self.state = "item"
        elif (self.state == "item") and (name in self.elements):
            self.state = name
        elif name == 'cc:license':
            self.items[-1].license = \
                attrs.get(u'rdf:resource', '').encode('utf-8')

    def endElement(self, name):
        if self.state and (name in self.elements):
            self.state = "item"

    def characters(self, contents):
        contents = contents.encode("utf-8")
        if (not self.state) or (self.state == "item"):
            pass
        elif self.state == "title":
            self.items[-1].title += contents
        elif self.state == "link":
            self.items[-1].link += contents
        elif self.state == "dc:date":
            self.items[-1].date += contents
        elif self.state == "dc:creator":
            self.items[-1].creator += contents
        elif self.state == "description":
            self.items[-1].description += contents
        elif self.state == "content:encoded":
            self.items[-1].content += contents

# End of RSShandler


def load_rss(filename):
    f = file(filename)
    handler = RSShandler()
    parse(f, handler)
    return handler.items


def error(msg='Something wrong'):
    print 'Content-Type: text/plain; charset=UTF-8'
    print
    print msg


def print_header(title=config['title']):
    title = cgi.escape(title, True)
    print '''Content-Type: text/html; charset=UTF-8

<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
<head>
  <meta http-equiv="content-type" content="text/html; charset=UTF-8" />
  <title>%s</title>
  <link rev="made" href="mailto:webmaster@fuktommy.com" />
  <link rel="contents" href="/blog" title="Top" />
  <link rel="stylesheet" type="text/css" href="/common.css" />
  <link rel="alternate" type="application/rss+xml" title="RSS" href="http://blog.fuktommy.com/xml/rss" />
  <link rel="meta" type="application/rdf+xml" title="license" href="/license" />
</head>
<body>
<h1>%s</h1>''' % (title, title)


def print_footer():
    print '''<address>&copy; 2006 <a href="mailto:webmaster@fuktommy.com">Fuktommy</a>.<br />
(<a rel="license" href="http://creativecommons.org/licenses/by/2.1/jp/" title="This work is licensed under a Creative Commons Attribution 2.1 Japan License.">Legal Notices</a>)
</address>
</body>
</html>'''


def convert_link(uri):
    if uri.startswith(config['link'] + '#e'):
        return basecgi + uri[len(config['link'])+2:]
    elif uri.startswith(config['link'] + '#a'):
        return basecgi + uri[len(config['link'])+2:]
    else:
        return uri


def print_index():
    archivefile = os.path.join(blogcgidir, config['archives'])
    print_header()
    print '<ul>'
    for i in load_rss(archivefile):
        link = convert_link(i.link)
        title = cgi.escape(i.title, True)
        print '<li><a href="%s">%s</a></li>' % (link, title)
    print '<li><a href="/">Return to Top</a></li>'
    print '</ul>'
    print_footer()


def print_archive(month):
    archivefile = os.path.join(blogcgidir,config['xmldir'],month,'index.rdf')
    if not os.path.exists(archivefile):
        error('%s is not exists' % month)
        return
    print_header(month)
    print '<ul>'
    for i in load_rss(archivefile):
        link = convert_link(i.link)
        title = cgi.escape(i.title, True)
        print '<li><a href="%s">%s</a></li>' % (link, title)
    print '<li><a href="/">Return to Top</a></li>'
    print '</ul>'
    print_footer()


class Entry:
    '''An article.'''

    def __init__(self, id):
        self.id = int(id)

    def month(self):
        return time.strftime('%Y-%m', time.localtime(self.id))

    def dirname(self):
        return os.path.join(blogcgidir, config['xmldir'], self.month())

    def path(self):
        return '%s/%d.rdf' % (self.dirname(), self.id)

    def load(self):
        path = self.path()
        if os.path.isfile(path):
            f = file(path)
            handler = RSShandler()
            parse(f, handler)
            self.item = handler.items[0]
            return True
        else:
            return False

# End of Entry


def print_entry(id):
    entry = Entry(id)
    ok = entry.load()
    if not ok:
        error('%s is not exists' % id)
        return
    print_header(entry.item.title)
    buf = entry.item.content
    data = ''
    while True:
        found = re.search(r'<a href=(["\'])([^"\']*)', buf)
        if not found:
            data += buf
            break
        data += buf[:found.start()] + '<a href=' + found.group(1) + \
                convert_link(found.group(2))
        buf = buf[found.end():]
    print data
    print '''<div style="text-align: center;">
<script type="text/javascript"><!--
  amazon_ad_tag = "fuktommysstor-22";  amazon_ad_width = "468";  amazon_ad_height = "60";  amazon_color_background = "EFEFEF";  amazon_color_link = "0000FF";//--></script>
<script type="text/javascript" src="http://www.assoc-amazon.jp/s/ads.js"></script>
</div>'''
    print '<ul>'
    date = re.sub(r'\+.*', '', entry.item.date).replace('T', ' ')
    print '<li>%s</li>' % date
    print '<li><a href="http://b.hatena.ne.jp/entry/%s">' % \
          urllib.quote(entry.item.link) + \
          '<img src="/b_entry_de.gif" width="16" height="12" alt="" />' + \
          ' Comments</a></li>'
    print '<li><a href="http://www.technorati.jp/search/search.html?' + \
          'queryMode=cosmos&amp;queryCosmos=%s&amp;language=all">' % \
          urllib.quote(entry.item.link) + \
          ' Links to entry Entry</a></li>'
    print '<li><a href="%s">Original Article (for PC)</a></li>' % \
          cgi.escape(entry.item.link, True)
    print '<li><a href="/">Return to Top</a></li>'
    print '</ul>'
    print_footer()


def main():
    try:
        os.environ['TZ'] = config['tz']
        time.tzset()
        query = os.environ.get('QUERY_STRING', '')
        found_archive = re.search(r'^(\d{4}-\d{2})$', query)
        found_entry = re.search(r'^(\d+)$', query)
        if found_archive:
            print_archive(found_archive.group(1))
        elif found_entry:
            print_entry(found_entry.group(1))
        else:
            print_index()
    except (KeyError, ValueError, IOError, OSError), e:
        error(e)

if __name__ == '__main__':
    main()
